<?php

namespace SGalinski\SgNews\Controller;

/***************************************************************
 *  Copyright notice
 *
 *  (c) sgalinski Internet Services (https://www.sgalinski.de)
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use SGalinski\SgNews\Service\HeaderMetaDataService;
use SGalinski\SgNews\Domain\Model\Category;
use SGalinski\SgNews\Domain\Model\News;
use TYPO3\CMS\Core\Page\PageRenderer;
use TYPO3\CMS\Frontend\Controller\TypoScriptFrontendController;

/**
 * Controller that handles the news single view page
 */
class SingleViewController extends AbstractController {
	/**
	 * @inject
	 * @var \SGalinski\SgNews\Domain\Repository\CategoryRepository
	 */
	protected $categoryRepository;

	/**
	 * @inject
	 * @var \SGalinski\SgNews\Domain\Repository\NewsRepository
	 */
	protected $newsRepository;

	/**
	 * @inject
	 * @var \TYPO3\CMS\Core\Charset\CharsetConverter
	 */
	protected $characterSetConverter;

	/**
	 * Renders the news single view
	 *
	 * @return void
	 */
	public function singleViewAction() {
		/** @var News $news */
		$currentId = (int) $GLOBALS['TSFE']->id;
		$news = $this->newsRepository->findByUid($currentId);
		if (!$news) {
			return;
		}

		/** @var Category $newsCategory */
		$newsCategory = $this->categoryRepository->findByUid($news->getPid());
		if (!$newsCategory) {
			return;
		}

		$similarNewsMetaData = [];
		$similarNews = $this->newsRepository->findRandomNews(3, $news);
		foreach ($similarNews as $similarNewsEntry) {
			/** @var News $similarNewsEntry */
			/** @var Category $category */
			$category = $this->categoryRepository->findByUid($similarNewsEntry->getPid());
			if (!$category) {
				continue;
			}

			$similarNewsMetaData[] = $this->getMetaDataForNews($similarNewsEntry, $category);
		}

		/** @var $typoScriptController TypoScriptFrontendController */
		$typoScriptController = $GLOBALS['TSFE'];

		/** @var PageRenderer $pageRenderer */
		$pageRenderer = $typoScriptController->getPageRenderer();

		$newsAuthor = $news->getAuthorFrontendUser();
		$newsMetaData = $this->getMetaDataForNews($news, $newsCategory);
		if ($newsMetaData['image']) {
			HeaderMetaDataService::addOgImageToHeader($newsMetaData['image']);
		} elseif ($newsMetaData['teaserImage']) {
			HeaderMetaDataService::addOgImageToHeader($newsMetaData['teaserImage']);
		}

		$this->view->assignMultiple(
			[
				'newsMetaData' => $newsMetaData,
				'similarNewsMetaData' => $similarNewsMetaData,
				'previousNews' => $this->newsRepository->findPreviousNewsEntryFromCurrentNews($news)->getFirst(),
				'nextNews' => $this->newsRepository->findNextNewsEntryFromCurrentNews($news)->getFirst(),
				'newsAuthor' => $newsAuthor,
			]
		);
	}
}

?>
